@@ -98,10 +98,11 @@ fit_hbd_pdr_parametric = function(	tree,
 
 	# set fit-control options, unless provided by the caller
 	if(is.null(fit_control)) fit_control = list()
-	if(is.null(fit_control$step.min)) fit_control$step.min = 0.001
-	if(is.null(fit_control$x.tol)) fit_control$x.tol = 1e-8
-	if(is.null(fit_control$iter.max)) fit_control$iter.max = 1000
-	if(is.null(fit_control$eval.max)) fit_control$eval.max = 2 * fit_control$iter.max * NFP
+	if(is.null(fit_control$step.min)) fit_control$step.min 	= 0.001
+	if(is.null(fit_control$x.tol)) fit_control$x.tol 		= 1e-10
+	if(is.null(fit_control$rel.tol)) fit_control$rel.tol 	= 1e-10
+	if(is.null(fit_control$iter.max)) fit_control$iter.max 	= 1000
+	if(is.null(fit_control$eval.max)) fit_control$eval.max 	= 2 * fit_control$iter.max * NFP
 		
 
 	################################
@@ -124,7 +125,7 @@ fit_hbd_pdr_parametric = function(	tree,
 			input_age_grid 	= age_grid;
 			input_PDRs		= PDRs
 		}
-		results = get_HBD_PDR_loglikelihood_CPP(branching_ages		= sorted_node_ages,
+		results = HBD_PDR_loglikelihood_CPP(branching_ages		= sorted_node_ages,
 												oldest_age			= oldest_age,
 												rholambda0			= input_rholambda0,
 												age_grid 			= input_age_grid,
